package service

import (
	"electricity_bill_calc/config"
	"electricity_bill_calc/global"
	"electricity_bill_calc/model"

	"xorm.io/builder"
	"xorm.io/xorm"
)

type _Meter04kVService struct{}

var Meter04kVService _Meter04kVService

func (_Meter04kVService) ListMeterDetail(park, keyword string, page int) ([]model.Meter04KV, int64, error) {
	cond := builder.NewCond()
	if len(keyword) > 0 {
		cond = cond.And(
			builder.Like{"address", keyword}.
				Or(builder.Like{"customer_name", keyword}).
				Or(builder.Like{"code", keyword}).
				Or(builder.Like{"contact_name", keyword}).
				Or(builder.Like{"contact_phone", keyword}))
	}
	total, err := global.DBConn.Where(cond).Count(new(model.Meter04KV))
	if err != nil {
		return make([]model.Meter04KV, 0), -1, err
	}
	var meters = make([]model.Meter04KV, 0)
	startItem := (page - 1) * config.ServiceSettings.ItemsPageSize
	err = global.DBConn.
		Where(cond).
		Limit(config.ServiceSettings.ItemsPageSize, startItem).
		Find(&meters)
	return meters, total, err
}

func (_Meter04kVService) Get04kVMeterDetail(park, code string) (*model.Meter04KV, error) {
	var meter = new(model.Meter04KV)
	has, err := global.DBConn.Where(builder.Eq{"code": code, "park_id": park}).NoAutoCondition().Get(meter)
	if err != nil {
		return nil, err
	}
	if !has {
		return nil, nil
	}
	return meter, nil
}

func (_Meter04kVService) insertNewMeter(tx *xorm.Session, meter model.Meter04KV) error {
	_, err := tx.Insert(meter)
	if err != nil {
		tx.Rollback()
	}
	return err
}

func (_Meter04kVService) updateMeter(tx *xorm.Session, meter model.Meter04KV) error {
	_, err := tx.
		Where(builder.Eq{"code": meter.Code, "park_id": meter.ParkId}).
		Cols("address", "customer_name", "contact_name", "contact_phone", "ratio", "seq", "public_meter", "dilute", "enabled").
		NoAutoCondition().
		Update(meter)
	if err != nil {
		tx.Rollback()
	}
	return err
}

func (m _Meter04kVService) CreateSingleMeter(meter model.Meter04KV) error {
	tx := global.DBConn.NewSession()
	if err := tx.Begin(); err != nil {
		return err
	}
	defer tx.Close()

	err := m.insertNewMeter(tx, meter)
	if err != nil {
		return err
	}
	err = tx.Commit()
	if err != nil {
		tx.Rollback()
		return err
	}
	return nil
}

func (m _Meter04kVService) UpdateSingleMeter(meter *model.Meter04KV) error {
	tx := global.DBConn.NewSession()
	if err := tx.Begin(); err != nil {
		return err
	}
	defer tx.Close()

	err := m.updateMeter(tx, *meter)
	if err != nil {
		return err
	}
	err = tx.Commit()
	if err != nil {
		tx.Rollback()
		return err
	}
	return nil
}
